/*
* TGPR: Transfer Learning Based Visual Tracking with Gaussian Process Regression
*
* Code to accompany the paper:
*   Transfer Learning Based Visual Tracking with Gaussian Process Regression
*   Jin Gao, Haibin Ling, Weiming Hu, Junliang Xing
*   ECCV2014, Part III, LNC 8691, pp. 188-203, 2014
*
* Copyright (C) 2014 Jin Gao
*
* This file is part of TGPR.
*
*
*The whole framework of this code is based on the framework of the publicly available code:
*
* Struck: Structured Output Tracking with Kernels
*
* Struck is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Struck is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Struck.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#ifndef CONFIG_H
#define CONFIG_H

#include <vector>
#include <string>
#include <ostream>

#define VERBOSE (0)
#define VERBOSEMY (0)
#define VERBOSEMY2 (0)
#define VERBOSEMY3 (0)
#define VERBOSEMY4 (0)
#define VERBOSEMY5 (0)
#include "omp.h"


class Config
{
public:
	Config() { SetDefaults(); }
	Config(const std::string& path);

	enum FeatureType
	{
		kFeatureTypeCovariance,
		kFeatureTypeHog,
		kFeatureTypeHaar,
		kFeatureTypeRaw,
		kFeatureTypeHistogram
	};

	enum KernelType
	{
		kKernelTypeBlocksimilarity,
		kKernelTypeLinear,
		kKernelTypeGaussian,
		kKernelTypeIntersection,
		kKernelTypeChi2
	};

	struct FeatureKernelPair
	{
		FeatureType feature;
		KernelType kernel;
		std::vector<double> params;
	};

	bool							quietMode;
	bool							debugMode;

	std::string						sequenceBasePath;
	std::string						sequenceName;
	std::string						resultsPath;

	int								frameWidth;
	int								frameHeight;

	int								Nblock;
	int								Nparticle;
	int								Threshold;
	int								interval;
	float							affisigx;
	float							affisigy;
	float							affisigscale;
	std::vector<FeatureKernelPair>	features;

	friend std::ostream& operator<< (std::ostream& out, const Config& conf);

private:
	void SetDefaults();
	static std::string FeatureName(FeatureType f);
	static std::string KernelName(KernelType k);
};

#endif