/*
* TGPR: Transfer Learning Based Visual Tracking with Gaussian Process Regression
*
* Code to accompany the paper:
*   Transfer Learning Based Visual Tracking with Gaussian Process Regression
*   Jin Gao, Haibin Ling, Weiming Hu, Junliang Xing
*   ECCV2014, Part III, LNC 8691, pp. 188-203, 2014
*
* Copyright (C) 2014 Jin Gao
*
* This file is part of TGPR.
*
*
*The whole framework of this code is based on the framework of the publicly available code:
*
* Struck: Structured Output Tracking with Kernels
*
* Struck is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Struck is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Struck.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#ifndef FEATURES_H
#define FEATURES_H

#include "Sample.h"

#define EIGEN_NO_DEBUG
#include <Eigen/Core>
#include <vector>
#include "omp.h"

class Features
{
public:
	Features();

	inline void Eval(const Sample& s, Eigen::VectorXd& featVect) const
	{
		//#pragma omp critical
		//{
			const_cast<Features*>(this)->UpdateFeatureVector(s,featVect);
		//	return m_featVec;
		//}
	}

	virtual void Eval(const MultiSample& s, std::vector<Eigen::VectorXd>& featVecs)
	{
		//Eigen::setNbThreads(4);
		// default implementation
		featVecs.resize(s.GetRects().size());
#pragma omp parallel for
		for (int i = 0; i < (int)featVecs.size(); ++i)
		{
			//featVecs[i] = Eval(s.GetSample(i));
			Eval(s.GetSample(i), featVecs[i]);
		}
		//Eigen::setNbThreads(1);
	}

	virtual int GetkNumBins() const
	{
		const_cast<Features*>(this)->GetkNumBinsFeatures();
		return kNumBinsFeatures;
	}

	inline int GetCount() const { return m_featureCount; }

protected:

	int m_featureCount;
	Eigen::VectorXd m_featVec;
	int kNumBinsFeatures;

	void SetCount(int c);
	virtual void UpdateFeatureVector(const Sample& s, Eigen::VectorXd& featVect) = 0;
	virtual void GetkNumBinsFeatures() = 0;

};

#endif
