/*
* TGPR: Transfer Learning Based Visual Tracking with Gaussian Process Regression
*
* Code to accompany the paper:
*   Transfer Learning Based Visual Tracking with Gaussian Process Regression
*   Jin Gao, Haibin Ling, Weiming Hu, Junliang Xing
*   ECCV2014, Part III, LNC 8691, pp. 188-203, 2014
*
* Copyright (C) 2014 Jin Gao
*
* This file is part of TGPR.
*
*
*The whole framework of this code is based on the framework of the publicly available code:
*
* Struck: Structured Output Tracking with Kernels
*
* Struck is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Struck is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Struck.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#ifndef GPR_H
#define GPR_H

#include "Rect.h"
#include "Sample.h"

#define EIGEN_NO_DEBUG
#include <vector>
#include <Eigen/Core>

#include <opencv/cv.h>
#include "omp.h"

class Config;
class Features;
class Kernel;
class Timing;

class GPR
{
public:
	GPR(const Config& conf, const Features& features, const Kernel& kernel, Timing& Timer);
	~GPR();

	virtual void Eval(const MultiSample& x, std::vector<int>& FIVE, bool& flagcase);
	virtual void UpdateT(const MultiSample& x, int y);
	virtual void UpdateA(const MultiSample& x, int y);

	virtual void Debug();
	virtual void costruSampleSet();
	virtual void costruKernel();

private:

	struct SupportPattern
	{
		std::vector<Eigen::VectorXd> x;
		std::vector<FloatRect> yv;
		std::vector<cv::Mat> images;
		int y;
		int refCount;
	};


	const Config& m_config;
	const Features& m_features;
	const Kernel& m_kernel;
	Timing* m_pTiming;

	std::vector<SupportPattern*> m_sps;
	SupportPattern* Ap_sps = new SupportPattern;
	SupportPattern* Tp_sps = new SupportPattern;
	SupportPattern* An_sps = new SupportPattern;
	SupportPattern* Tn_sps = new SupportPattern;
	SupportPattern* Unlabeled_sps = new SupportPattern;

	SupportPattern* SampleSet_sps = new SupportPattern;

	cv::Mat m_debugImage;
	cv::Mat m_debugImage1;
	cv::Mat m_debugImage2;

	Eigen::MatrixXd m_K;
	int flag;
	int kNumBins;
	cv::Mat Index;
	cv::Mat Index1;
	void BudgetMaintenanceGPR();
	void BudgetMaintenanceRemoveGPR();

	void UpdateDebugImage();
	void UpdateDebugImage1();
	void UpdateDebugImage2();
	Eigen::MatrixXd ContextLogSimi0(const std::vector<Eigen::VectorXd> x);
	Eigen::MatrixXd ComputInvers(const Eigen::MatrixXd M);
	Eigen::MatrixXd SetZerosInW(const Eigen::MatrixXd W, int knn);
	double GetMinKth(double* ary, int lowIndex, int highIndex, int k);
	int PartitionMin(double* ary, int lowIndex, int highIndex);
	void Swap(double& x, double& y);
};

#endif
