/*
* TGPR: Transfer Learning Based Visual Tracking with Gaussian Process Regression
*
* Code to accompany the paper:
*   Transfer Learning Based Visual Tracking with Gaussian Process Regression
*   Jin Gao, Haibin Ling, Weiming Hu, Junliang Xing
*   ECCV2014, Part III, LNC 8691, pp. 188-203, 2014
*
* Copyright (C) 2014 Jin Gao
*
* This file is part of TGPR.
*
*
*The whole framework of this code is based on the framework of the publicly available code:
*
* Struck: Structured Output Tracking with Kernels
*
* Struck is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Struck is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Struck.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#ifndef HAAR_FEATURES_H
#define HAAR_FEATURES_H

#include "Features.h"
#include "HaarFeature.h"

#include <vector>
#include "omp.h"

class Config;

class HaarFeatures : public Features
{
public:
	HaarFeatures(const Config& conf);

private:
	std::vector<HaarFeature> m_features;

	virtual void UpdateFeatureVector(const Sample& s, Eigen::VectorXd& featVect);
	virtual void GetkNumBinsFeatures();

	void GenerateSystematic();
};

#endif
