/*
* TGPR: Transfer Learning Based Visual Tracking with Gaussian Process Regression
*
* Code to accompany the paper:
*   Transfer Learning Based Visual Tracking with Gaussian Process Regression
*   Jin Gao, Haibin Ling, Weiming Hu, Junliang Xing
*   ECCV2014, Part III, LNC 8691, pp. 188-203, 2014
*
* Copyright (C) 2014 Jin Gao
*
* This file is part of TGPR.
*
*
*The whole framework of this code is based on the framework of the publicly available code:
*
* Struck: Structured Output Tracking with Kernels
*
* Struck is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Struck is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Struck.  If not, see <http://www.gnu.org/licenses/>.
*
*/
#ifndef IMAGE_REP_H
#define IMAGE_REP_H

#include "Rect.h"

#include <opencv/cv.h>
#include <vector>
#define EIGEN_NO_DEBUG
#include <Eigen/Core>
#include "omp.h"

class ImageRep
{
public:
	ImageRep(const cv::Mat& rImage, bool computeIntegral, bool computeIntegralHists, bool computeIntegralCov, bool colour = false);

	int Sum(const IntRect& rRect, int channel = 0) const;
	void Hist(const IntRect& rRect, Eigen::VectorXd& h) const;
	cv::Mat Cov(const IntRect& rRect) const;

	inline const cv::Mat& GetImage(int channel = 0) const { return m_images[channel]; }
	inline const IntRect& GetRect() const { return m_rect; }

private:
	std::vector<cv::Mat> m_images;
	std::vector<cv::Mat> m_integralImages;
	std::vector<cv::Mat> m_integralHistImages;
	std::vector<cv::Mat> m_integralCovP;
	std::vector<cv::Mat> m_integralCovQ;
	std::vector<cv::Mat> CovFeatureVector;
	int m_channels;
	//int numCovFeature;
	enum ConvolutionType {   /* Return the full convolution, including border */
		CONVOLUTION_FULL,    /* Return only the part that corresponds to the original image */
		CONVOLUTION_SAME,   /* Return only the submatrix containing elements that were not influenced by the border */
		CONVOLUTION_VALID
	};
	void conv2(const cv::Mat &img, const cv::Mat& kernel, ConvolutionType type, cv::Mat& dest) const;
	cv::Mat ToPVector(const int x, const int y) const;
	cv::Mat ToQMatrix(const int x, const int y) const;
	IntRect m_rect;

};

#endif
