/*
* TGPR: Transfer Learning Based Visual Tracking with Gaussian Process Regression
*
* Code to accompany the paper:
*   Transfer Learning Based Visual Tracking with Gaussian Process Regression
*   Jin Gao, Haibin Ling, Weiming Hu, Junliang Xing
*   ECCV2014, Part III, LNC 8691, pp. 188-203, 2014
*
* Copyright (C) 2014 Jin Gao
*
* This file is part of TGPR.
*
*
*The whole framework of this code is based on the framework of the publicly available code:
*
* Struck: Structured Output Tracking with Kernels
*
* Struck is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Struck is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Struck.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#include "MultiFeatures.h"

using namespace Eigen;
using namespace std;

MultiFeatures::MultiFeatures(const vector<Features*>& features) :
m_features(features)
{
	int d = 0;
	for (int i = 0; i < (int)features.size(); ++i)
	{
		d += features[i]->GetCount();
	}
	SetCount(d);
}

void MultiFeatures::GetkNumBinsFeatures()
{
	kNumBinsFeatures = m_features.size();
}

void MultiFeatures::UpdateFeatureVector(const Sample& s, Eigen::VectorXd& featVect)
{
	int start = 0;
	featVect = Eigen::VectorXd::Zero(m_featVec.size());
	for (int i = 0; i < (int)m_features.size(); ++i)
	{
		int n = m_features[i]->GetCount();
		Eigen::VectorXd featVect_tmp = Eigen::VectorXd::Zero(n);
		m_features[i]->Eval(s, featVect_tmp);
		featVect.segment(start, n) = featVect_tmp;
		start += n;
	}
}
