/*
* TGPR: Transfer Learning Based Visual Tracking with Gaussian Process Regression
*
* Code to accompany the paper:
*   Transfer Learning Based Visual Tracking with Gaussian Process Regression
*   Jin Gao, Haibin Ling, Weiming Hu, Junliang Xing
*   ECCV2014, Part III, LNC 8691, pp. 188-203, 2014
*
* Copyright (C) 2014 Jin Gao
*
* This file is part of TGPR.
*
*
*The whole framework of this code is based on the framework of the publicly available code:
*
* Struck: Structured Output Tracking with Kernels
*
* Struck is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Struck is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Struck.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#include "RawFeatures.h"
#include "Config.h"
#include "Sample.h"
#include "Rect.h"

#include <iostream>

using namespace Eigen;
using namespace cv;

static const int kPatchSize = 16;

RawFeatures::RawFeatures(const Config& conf) :
m_patchImage(kPatchSize, kPatchSize, CV_8UC1)
{
	SetCount(kPatchSize*kPatchSize);
}

void RawFeatures::GetkNumBinsFeatures()
{
	kNumBinsFeatures = kPatchSize*kPatchSize;
}

void RawFeatures::UpdateFeatureVector(const Sample& s, Eigen::VectorXd& featVect)
{
	IntRect rect = s.GetROI(); // note this truncates to integers
	cv::Rect roi(rect.XMin(), rect.YMin(), rect.Width(), rect.Height());
	cv::resize(s.GetImage().GetImage(0)(roi), m_patchImage, m_patchImage.size());
	//equalizeHist(m_patchImage, m_patchImage);
	featVect = Eigen::VectorXd::Zero(m_featVec.size());
	int ind = 0;
	for (int i = 0; i < kPatchSize; ++i)
	{
		uchar* pixel = m_patchImage.ptr(i);
		for (int j = 0; j < kPatchSize; ++j, ++pixel, ++ind)
		{
			featVect[ind] = ((double)*pixel) / 255;
		}
	}
}
