/*
* TGPR: Transfer Learning Based Visual Tracking with Gaussian Process Regression
*
* Code to accompany the paper:
*   Transfer Learning Based Visual Tracking with Gaussian Process Regression
*   Jin Gao, Haibin Ling, Weiming Hu, Junliang Xing
*   ECCV2014, Part III, LNC 8691, pp. 188-203, 2014
*
* Copyright (C) 2014 Jin Gao
*
* This file is part of TGPR.
*
*
*The whole framework of this code is based on the framework of the publicly available code:
*
* Struck: Structured Output Tracking with Kernels
*
* Struck is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Struck is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Struck.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#ifndef TRACKER_H
#define TRACKER_H

#include "Rect.h"
#define EIGEN_NO_DEBUG
#include <vector>
#include <Eigen/Core>
#include <opencv/cv.h>
#include "omp.h"

class Config;
class Features;
class Kernel;
class ImageRep;
class GPR;
class Timing;

class Tracker
{
public:
	Tracker(const Config& conf, Timing& Timer);
	~Tracker();

	void InitialiseGPR(const cv::Mat& frame, FloatRect bb, const int& frameInd);
	void Reset();
	void Track(const cv::Mat& frame, const int& frameInd);
	void Debug();

	inline const FloatRect& GetBB() const { return m_bb; }
	inline bool IsInitialised() const { return m_initialised; }

private:
	const Config& m_config;
	bool m_initialised;
	std::vector<Features*> m_features;
	std::vector<Kernel*> m_kernels;
	GPR* m_pclassifier;
	Timing* m_pTiming;
	FloatRect m_bb;
	bool m_needsIntegralImage; //indicate whether using Haar feature or not;
	bool m_needsIntegralHist; //indicate whether using Histogram feature or not;
	bool m_needsIntegralCov; //indicate whether using Covariate feature or not;
	bool flagcase; //indicate whether increasing the parameters of particle generation;
	float width_initial, height_initial;

	void UpdateSampleSet(const ImageRep& image, const int& frameInd);
	FloatRect meanRect(const std::vector<FloatRect>& keptRects, const std::vector<int>& FIVE);
};

#endif
